datagenkv <- function(av,bv=NULL,interac=NULL,nrep=5,rlen=129,sigma=1,seed=NULL){
#
# Function to generate examples of artificial data in the form of numeric
# vectors that are designed to "look like" K-functions.
#
if(!requireNamespace("spatstat.geom"))
   stop("Required package \"spatstat.geom\" is not available.\n")

if(is.null(seed)) seed <- sample(1:1e5,1)
a <- length(av)
if(is.null(bv)) {
    b <- 1
} else {
    b <- length(bv)
    if(is.null(interac)) {
        interac <- 0
    } else {
        if(length(interac) != a*b) {
            whinge <- paste0("Length of \"interac\" is ",length(interac),
                             "; should be ",a*b,"\n")
            stop(whinge)
        }
        interac <- rep(interac,nrep)
    }
}

if(is.null(bv)) {
    A  <- factor(letters[1:a])
    B  <- NULL
    mn <- av
} else {
    A  <- factor(rep(letters[1:a],b))
    B  <- factor(rep(1:b,each=a))
    mn <- as.vector(outer(av,bv,"+")) + interac
    if(length(sigma) == 1) {
        sigma <- rep(sigma,b)
    } else if(length(sigma) != b) {
        stop(paste0("The length of \"sigma\" must be either 1 or ",b,".\n"))
    }
    names(sigma) <- levels(B)
    names(mn)    <- B
}
rrr   <- if(rlen>1) seq(0,0.25,length=rlen) else 1
kbase <- if(rlen>1) rrr^2 else 0
Brep  <- if(is.null(bv)) NULL else rep(B,nrep)
rep   <- rep(1:nrep,each=length(B))
kays  <- lapply(1:(a*b*nrep),function(k,Brep,sigma,mn,rlen,kbase){
                ss    <- if(is.null(Brep)) sigma else sigma[Brep[k]]
                ss    <- ss*(if(rlen>1) pmax(kbase,1) else 1)
                ml    <- length(mn)
                ko    <- if(k%%ml == 0) ml else k%%ml
                noise <- stats::rnorm(rlen,0,ss)
                xx    <- noise+mn[ko]+kbase
                pmax(xx,0)
                },Brep=Brep,sigma=sigma,mn=mn,rlen=rlen,kbase=kbase)
rslt <- spatstat.geom::hyperframe(y=kays,A=rep(A,nrep),B=rep(B,nrep),
                                  rep=rep)
if(rlen > 1) attr(rslt,"r") <- rrr
attr(rslt,"seed") <- seed
rslt
}
