\name{kanova}
\alias{kanova}
\title{
    Pseudo analysis of variance of K-functions
}
\description{
    One-way and two-way pseudo analysis of variance for replicated
    point patterns, grouped by one or two classification factors,
    on the basis of the values of a summary function.  This summary
    function may be specified by the user.  It is usually one of
    the four standard summary functions, most often the K-function,
    which is the default.
}
\usage{
    kanova(fmla,data,expo=2,rsteps=128,r=NULL,sumFnNm=NULL,
           warnSFN=TRUE,test=TRUE,permtype=c("stdres","data"),
           nperm=99,brief=TRUE,verb=TRUE,keepdata=FALSE)
}
\arguments{
  \item{fmla}{A formula specifying the model to be fitted.
  There can be at most two main effect predictors (possibly with
  interaction between them).  The left hand side of \code{fmla}
  may be omitted.  If so, it is taken to be the name of the first
  column of \code{data}.
}
  \item{data}{A hyperframe (see
  \code{\link[spatstat.geom]{hyperframe}()}) containing the data to
  be analysed.  If the left hand side of \code{fmla} is supplied,
  then \code{data} must have a column with a name which matches
  this left hand side. If no such column exists, an error is thrown.
  The response column may be a list of point patterns, or it may be
  a list of numeric vectors (all of which must have the same length).

  These numeric vectors are notionally values of a diagnostic
  or summary function, applied to point patterns, and then
  evaluated at the attribute \code{"r"} (see below) of \code{data}.
  However that the entries in the list need not actually \emph{be}
  summary functions.  They are simply numeric vectors and \emph{not}
  objects of class \code{"fv"} as returned by a summary function.
  In particular they may be scalars, which allows the \code{kanova()}
  function to be applied to analysis of variance of scalars.

  Argument \code{data} may also have one or two columns with names
  matching those of the main effect predictors as specified by
  \code{fmla}.  If such columns cannot be found, then objects
  with these names are sought in the \dQuote{parent frame}
  (see \code{\link{parent.frame}()}.)  If the appropriate objects
  still cannot be found, then an error is thrown.  These columns
  (or objects) are internally coerced to factors.

  If the response consists of numeric vectors, and if these vectors
  are \emph{not} scalars, then \code{data} must have an attribute
  \code{"r"}  which specifies the \dQuote{argument} of the pseudo
  summary functions in the list.  This attribute should be an
  numeric vector and must be of the same length as the numeric
  vectors in the list.  If the response functions are scalars,
  then no \code{"r"} attribute is needed, and, if present, is ignored.

  Finally, \code{data} may have a columns named \code{"wts"}
  (\dQuote{weights}) whose entries are positive scalars.  Notionally
  they may be thought of as reciprocals of a power of the numbers of
  points in the corresponding point patterns.  If there is no column
  named \code{"wts"} then the weights are all taken to equal 1.

  If the response column consists of a list of point patterns then
  any column named \code{"wts"} is ignored, and the weights are
  indeed reciprocals of a power of the numbers of points in the
  corresponding point patterns.  The impact of using these weights
  (when \code{expo}, see below, is strictly positive) is to diminish
  the influence of quantities corresponding to patterns having few
  points, and conversely to emphasise the influence of quantities
  corresponding to patterns having many points.
}
  \item{expo}{Non-negative numeric scalar.  Ignored unless the
  response is a list of point patterns.  Statistics in \code{kanova}
  are in general calculated using \emph{weighted} means and in this
  context the weights are the reciprocals of the counts of points in
  the patterns, raised to the power \code{expo}.  To use unweighted
  means, set \code{expo=0}.  If \code{expo} is equal to \code{1}
  then the weights are simply the reciprocals of the pattern counts.
}
  \item{rsteps}{Integer scalar.  Ignored if argument \code{r}
  (see below) is supplied (i.e. is not \code{NULL}), or if the
  response consists of a list of numeric vectors (rather than a
  list of point patterns). The argument \code{rsteps} specifies the
  number of (equal) steps between values of the vector \code{r} at
  which the summary function is evaluated.  The values of \code{r}
  are equispaced on the interval from 0 to \code{rtop}, the latter
  being calculated internally.  The value of \code{rtop} depends
  on the observation windows of the patterns in the response and
  on their intensities.  It also depends on the summary function
  being used.
}
  \item{r}{Numeric vector.  Ignored unless the response consists of
  a list of point patterns.  Note that if the response consists of
  a list of vectors, representing notional diagnostic functions,
  then argument \code{data} must have an attribute \code{r} to
  provide the relevant \code{r} object.  In this setting an error
  is thrown if the required attribute is not present (unless the
  response vectors are in fact scalars).

  The argument \code{r} specifies the values (distances) at
  which the summary/diagnostic function (see below) is evaluated.
  Errors may be thrown if the first entry of \code{r} is not 0,
  or if the values of \code{r} are not sufficiently finely spaced.
  Generally users should not specify \code{r}.  unless they have
  a sound understanding of what they are doing.
}
  \item{sumFnNm}{Character string naming the summary/diagnostic
  function to be used.  If this is not one of the \dQuote{standard
  four}, i.e. \code{"Kest"}, \code{"Fest"},  \code{"Gest"}, or
  \code{"Jest"},and if \code{warnSFN} (see below) is \code{TRUE},
  then a warning is issued.  In this case \bold{there may
  be problems}; the code is not robust in this respect.
  If \code{sumFnNm} is not specified (left \code{NULL}) then it
  defaults to \code{"Kest"}.  Users should apply summary functions
  other than \code{Kest()} only if they have a sound understanding
  of what they are doing.  See \bold{Notes}.
}
  \item{warnSFN}{Logical scalar.  Should a warning be issued if
  \code{sumFnNm} is not one of the \dQuote{standard four}?
}
  \item{test}{Logical scalar.  Should a Monte Carlo test of the
  null hypothesis be carried out?
} 
  \item{permtype}{Character string specifying what sort of
  permutations should be done to produce the Monte Carlo test
  statistics.  Ignored if \code{test} is \code{FALSE}.

  If \code{permtype} is \code{"stdres"} then the Monte Carlo data
  are formed by permuting the residuals, from the \emph{saturated}
  model, which have been standardised by dividing them by their
  (estimated) standard deviations.  After permutation, the residuals
  are \dQuote{unstandardised} by multiplying them by the appropriate
  standard deviation.  They are then added back to the fitted values
  from the fitted model.

  If \code{permtype} is \code{"data"} then the Monte Carlo data
  are formed by permuting the original data sets. 

  In the two-way setting, when the test is for the main effect A,
  then the data (if \code{permtype} is \code{"data"}) are permuted
  \emph{within} the levels of B.

  If \code{fmla} specifies an  interaction between the main effect
  predictors, then \code{permtype} cannot be \code{"data"} and
  \emph{must} be \code{"stdres"}, otherwise an error is thrown.
}
  \item{nperm}{
  The number of permutations to be used to determine the Monte Carlo
  \eqn{p}-value.  Ignored if \code{test} is \code{FALSE}
}
  \item{brief}{Logical scalar.  Should the object returned by this
  function be \dQuote{brief}?  See \bold{Value}.
}
  \item{verb}{
  I.e. \dQuote{verbose}.  Logical scalar.  Should rudimentary
  \dQuote{progress reports} be printed out (in the course of
  conducting the permutation test for \dQuote{significance} of
  the test statistic)?  Such \dQuote{reports} consists simply of
  indications of how many permutations have been effected so far.
  Ignored if \code{test} is \code{FALSE}.
}
  \item{keepdata}{Logical scalar.  Should a copy of the data, to
  which the model has been fitted, be included as a component of
  of the object returned by \code{kanova()}?  See \bold{Value}
  for more detail.
}
}
\details{
  The formulae used in the \code{fmla} argument \dQuote{look like}
  those used in \dQuote{ordinary} analysis of variance, but have a
  different interpretation.  The formula \code{y ~ A + B} does not
  actually fit the additive \code{A + B} model.  It effects as test
  of the \dQuote{significance} of A, allowing for B.  The factor B is
  ``allowed for'' (if \code{permtype} is \code{"data"}) by permuting
  the data \emph{within} the levels of B.  If \code{permtype}
  is \code{"stdres"}) the standardised residuals are permuted.
  These are residuals from the saturated model, which includes a
  B effect.  Likewise \code{y ~ B + A} tests for a B effect,
  allowing for an A effect, in the same manner.

  In particular \code{y ~ A + B} is not equivalent to \code{y ~
  B + A} (whereas in \dQuote{ordinary} analysis of variance) they
  \emph{are} equivalent).

  The formula \code{y ~ A * B} tests the model with interaction
  against the additive model, as in ordinary analysis of variance.
  The additive model is usually not meaningful in the current
  context, so a test for interaction is probably not meaningful
  either.  Such a test is included in the code basically for the
  sake of completeness, and to allow for the possibility that a
  user may encounter a circumstance in which the additive model
  actually is meaningful.

  The value of the test statistic is obtained as a sum of numerical
  integrals of certain sums of squares.  The integrals are
  computed via a rough trapezoid rule and the sums of squares are
  \dQuote{standardised}.  I.e. they are  divided by the estimated
  variance of the quantity being squared.  This procedure is
  analogous to the studentisation procedure used by Hahn, 2012. The
  quantities that are involved in the sums of squares are formed
  from certain \dQuote{fitted values} which are weighted means
  of the observations.  The variance referred to is formed as a
  weighted mean of squares of the residuals, which are of course
  equal to the observations minus the fitted values.

  The integration is carried out over the value of \code{r}, the
  argument of the summary functions that are being analysed.  If the
  response consists of numeric vectors of length 1, i.e. of scalars,
  then no integration is in fact performed, and the corresponding
  (downweighted) sum of squares is returned.  You may, if you like,
  think of this as integrating with respect to a measure which has
  a point mass of 1 at a conceptual single value of \code{r}.

  More detail about the test statistic, the fitted values, the
  residuals and the estimated variance, can be found in the vignette
  \code{"testStat"}.
}

\section{Notes}{
  \itemize{
  \item Simulation experiments have given some evidence that
  \code{Fest()} and \code{Gest()} and \code{Jest()} lead to tests
  that have lower power than that obtained than tests obtained by
  using \code{Kest()}.  The power obtained seems to be substantially
  lower in the case of \code{Fest()} and \code{Gest()}, somewhat
  lower in the case of \code{Jest()}.

  Consequently, users are advised to eschew the use of \code{Fest()},
  \code{Gest()} and \code{Jest()} (despite their ready availability)
  and to stick with the default summary function \code{Kest()}.
  Users should ignore this advice \emph{only} if they have a sound
  reason for doing so and a sound understanding of the consequences.

  \item Only one-way and two-way (pseudo) analyses of variance
  are accommodated.  If you feel inclined to ask why there is not
  provision for higher order analysis of variance, just look at
  the code and the answer should be obvious.  It \emph{might} be
  possible to implement higher order pseudo analysis of variance of
  summary functions, but this is unlikely to have any practical use.
  Writing the code would, for me, be a nightmare!
  }
}
\value{If \code{brief} is \code{TRUE}, the object returned is
  a list with components:

  \item{Effectname}{Character string naming the effect being tested for.}

  \item{stat}{Numerical scalar equal to the value of the test statistic
  calculated from the original data.}

  \item{pvalue}{The Monte Carlo \eqn{p}-value of the test calculated
  by comparing \code{stat} with test statistics formed from simulated
  data, generated by permutation, which satisfy the null hypothesis.}

  If \code{brief} is \code{FALSE} the list returned has additional
  components:

  \item{nperm}{The \code{nperm} argument.}

  \item{permtype}{The \code{permtype} argument.}

  \item{Tstar}{The vector of \code{nperm} values of the test statistic
  calculated from the simulated data sets.}

  \item{fmla}{The \code{fmla} argument.}

  \item{sumFnNm}{The \code{sumFnNm} argument.}

  \item{data}{The \code{data} argument.  This may possibly have
  been augmented by any predictor values which were not found in
  the original \code{data} and were located in the parent frame.}

  Components \code{pvalue}, \code{nperm} and \code{Tstar} are
  present only if \code{test} is \code{TRUE}.  Component \code{data}
  is present only if \code{keepdata} is \code{TRUE}.
}
\references{
Diggle, Peter J., Mateu, Jorge and Clough, Helen E. (2000) A
comparison between parametric and non-parametric approaches to the
analysis of replicated spatial point patterns, \emph{Advances in
Applied Probability} \bold{32}, pp. 331 -- 343.

Diggle, P. J., Lange, N. and Benes, F. (1991) Analysis of variance
for replicated spatial point patterns in clinical neuroanatomy,
\emph{Journal of the American Statistical Association}, \bold{86},
pp. 618 -- 625.

Hahn, Ute (2012) A studentized permutation test for the
comparison of spatial point patterns, \emph{Journal of the
American Statistical Association}, \bold{107}, pp. 754 -- 764,
DOI: 10.1080/01621459.2012.688463.
}
\author{Rolf Turner
  \email{rolfturner@posteo.net}
}

\seealso{
\code{\link[spatstat.explore]{studpermu.test}()}
}

\examples{
# The following is inappropriate since there is a second
# classification factor.
set.seed(104)
s1 <- kanova(patterns ~ Pos,data=stomata,permtype="d")

# Here we are testing for a Layer effect allowing for a Pos effect.
set.seed(7)
s2a <- kanova(patterns ~ Layer + Pos, data=stomata,permtype="d")
s2b <- kanova(patterns ~ Layer + Pos, data=stomata,permtype="s")

# Here we are testing for a Pos effect allowing for a Layer effect.
set.seed(78)
s3a <- kanova(patterns ~ Pos + Layer, data=stomata)
# permtype defaults to "stdres".
\dontrun{ # Takes too long.
    set.seed(24)
    s3b <- kanova(patterns ~ Pos + Layer, data=stomata,nperm=999)
    # Get a p-value of 0.001
}

# Here, we are testing for a Layer by Pos interaction.  Probably
# not meaningful.
set.seed(2)
s4 <- kanova(patterns ~ Layer * Pos, data=stomata) # permtype must be "s"

# Artificial data.
\dontrun{
if(!requireNamespace("spatstat.geom"))
    stop("Required package \"spatstat.geom\" is not available.\n")
set.seed(3)
r    <- seq(0,25,length=129)
rsp  <- lapply(1:144,function(k){pi*r^2 + runif(129,-0.1,0.1)})
rsp  <- lapply(rsp,function(x){pmax(0,x)})
fac1 <- factor(rep(1:4,12,each=3))
fac2 <- factor(rep(1:3,48))
wts  <- sample(50:100,144,replace=TRUE)
X    <- spatstat.geom::hyperframe(rsp=rsp,fac1=fac1,fac2=fac2,wts=wts)
attr(X,"r") <- r
set.seed(118)
# Testing for a fac1 effect, allowing for a fac2 effect.
s5a  <- kanova(rsp ~ fac1 + fac2, data=X,brief=FALSE)
# Testing for an interaction; meaningful in this artificial data
# context.
s5b  <- kanova(rsp ~ fac1*fac2,data=X,brief=FALSE)
}

# Scalar data.
\dontrun{
if(require(Devore7)) {
    X   <- spatstat.geom::as.hyperframe(xmp11.10)
    s6a <- kanova(Tempr ~ Period*Strain,data=X,nperm=999)
    s6b <- kanova(Tempr ~ Period+Strain,data=X,nperm=999)
    s6c <- kanova(Tempr ~ Strain+Period,data=X,nperm=999)
    chk <- lm(Tempr ~ Period*Strain,data=X)
# anova(chk) reveals p-values that are at least roughly
# similar to those in s6a, s6b, and s6c.
} else {
    stop("Required package \"Devore7\" is not available.\n")
}
}
}

\keyword{ htest }
